@@ -18,6 +18,7 @@
 #include <glade/glade.h>
 #include <gnome.h>
 #include <gconf/gconf-client.h>
+#include <glib/gstdio.h>
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -36,12 +37,22 @@ Preferences gwget_pref;
 static void 
 show_prefered_columns(void);
 
+static gboolean
+gwget_destination_file_exists(GwgetData *data);
+
 static void 
 gwget_gconf_notify_toolbar(GConfClient *client,
 			   guint        cnxn_id,
 		   	   GConfEntry  *entry,
 		   	   gpointer     user_data);
 
+static gboolean
+view_selection_func (GtkTreeSelection *selection,
+                        GtkTreeModel     *model,
+                        GtkTreePath      *path,
+                        gboolean          path_currently_selected,
+                        gpointer          userdata);
+
 void 
 main_window(void) 
 {
@@ -71,6 +82,8 @@ main_window(void)
 	gtk_tree_view_set_model(GTK_TREE_VIEW(treev),GTK_TREE_MODEL(model));
 	select = gtk_tree_view_get_selection (GTK_TREE_VIEW (treev));
 	gtk_tree_selection_set_mode (select, GTK_SELECTION_SINGLE);
+	/* Set a callback when a selection is made for update the menu Download */
+	gtk_tree_selection_set_select_function(select, view_selection_func, NULL, NULL);
 
 	/*
 	 * Update window size in gconf on resize 
@@ -82,7 +95,7 @@ main_window(void)
 	
 	/* add the columns titles to the tree view */
 	add_columns (GTK_TREE_VIEW (treev));
-	
+
 	gconf_client = gconf_client_get_default();
 	gconf_client_add_dir (gconf_client, 
 			      "/apps/gwget2", 
@@ -136,16 +149,30 @@ main_window(void)
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml,"view_toolbar")),gwget_pref.view_toolbar);
 	toolbar = glade_xml_get_widget(xml,"bonobotoolbar"); 
 	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_toolbar");
+	if (gwget_pref.trayonly) {
+		gtk_widget_show(GTK_WIDGET(window));
+		gtk_widget_hide(GTK_WIDGET(window));
+	}
+	else 
+		gtk_widget_show(GTK_WIDGET(window));
+
 	if (!gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item))) {
 		gtk_widget_hide(GTK_WIDGET(toolbar));
 	} else {
 		gtk_widget_show(GTK_WIDGET(toolbar));
 	}
 
-	if (gwget_pref.trayonly)
-		gtk_widget_hide(GTK_WIDGET(window));
-	else 
-		gtk_widget_show(GTK_WIDGET(window));
+	/* Show the statusbar ? */
+	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(glade_xml_get_widget(xml,"view_statusbar")), gwget_pref.view_statusbar);
+	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_statusbar");
+	if (gwget_pref.view_statusbar)
+	{
+		gtk_widget_show (GTK_WIDGET(glade_xml_get_widget(xml, "statusbar")));
+	} else {
+		gtk_widget_hide (GTK_WIDGET(glade_xml_get_widget(xml, "statusbar")));
+	}
+		
+
 	
 	systray_load();
 
@@ -158,19 +185,49 @@ main_window(void)
 		
 }
 
-gboolean
+static 
+gboolean view_selection_func (GtkTreeSelection *selection,
+			GtkTreeModel     *model,
+			GtkTreePath      *path,
+			gboolean          path_currently_selected,
+			gpointer          userdata)
+{
+	GtkTreeIter iter;
+
+	g_return_val_if_fail(gwget_data_get_selected()!=NULL, TRUE);
+
+	if (gtk_tree_model_get_iter(model, &iter, path))
+	{
+		gchar *name;
+
+		gtk_tree_model_get(model, &iter, FILENAME_COLUMN, &name, -1);
+
+		if (!path_currently_selected)
+		{
+			gwget_data_set_menus (gwget_data_get_selected());
+		}
+		g_free(name);
+	}
+
+	return TRUE; /* allow selection state to change */
+}
+
+
+
+
+static gboolean
 gwget_destination_file_exists(GwgetData *data)
 {
 	struct stat s;
 	
-	//g_print("%s %d\n",data->local_filename , g_stat(data->local_filename,&s));	
 	return ( g_stat(data->local_filename,&s) == 0 );
 }
 
 void 
 gwget_get_defaults_from_gconf(void)
 {
-	gint num_dl, i, total_size;
+	gint num_dl, i;
+	gint64 total_size;
 	GwgetData *data;
 	gchar *key,*url,*dir,*name;
 	DlState state;
@@ -198,6 +255,7 @@ gwget_get_defaults_from_gconf(void)
 	gwget_pref.view_elapse_time=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_elapse_time",NULL);
 	gwget_pref.view_down_speed=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_down_speed",NULL);
 	gwget_pref.view_toolbar=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_toolbar",NULL);
+	gwget_pref.view_statusbar=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_statusbar",NULL);
 	gwget_pref.view_file_type=gconf_client_get_bool(gconf_client,"/apps/gwget2/view_file_type",NULL);
 	gwget_pref.limit_speed = gconf_client_get_bool (gconf_client,"/apps/gwget2/limit_speed", NULL);
 	gwget_pref.max_speed=gconf_client_get_int(gconf_client,"/apps/gwget2/max_speed",NULL);
@@ -234,8 +292,8 @@ gwget_get_defaults_from_gconf(void)
 		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/state",i);
 		state=gconf_client_get_int(gconf_client,key,NULL); 
 		
-		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/total_size",i);
-		total_size = gconf_client_get_int (gconf_client, key, &error);
+		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/file_size",i);
+		total_size = atoll(gconf_client_get_string (gconf_client, key, &error));
 		if (!error) {		
 			gwget_data_set_total_size (data, total_size);
 		} else { 
@@ -363,6 +421,7 @@ add_columns (GtkTreeView *treeview)
 	
 	gtk_tree_view_column_set_title (GTK_TREE_VIEW_COLUMN(column),_("File Name"));
 	
+	gtk_tree_view_column_set_sort_column_id (column, FILENAME_COLUMN);
 	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
@@ -374,6 +433,7 @@ add_columns (GtkTreeView *treeview)
 							  STATE_COLUMN,
 							  NULL);
 	gtk_tree_view_column_set_resizable(column, TRUE); 
+	gtk_tree_view_column_set_sort_column_id (column, STATE_COLUMN);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 	gtk_tree_view_append_column (treeview, column);
 	
@@ -386,6 +446,7 @@ add_columns (GtkTreeView *treeview)
 							NULL);
 
 	gtk_tree_view_column_set_sort_column_id (column, CURRENTSIZE_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
   	gtk_tree_view_append_column (treeview, column);
 	
 	/* Total Size */
@@ -396,6 +457,7 @@ add_columns (GtkTreeView *treeview)
 							TOTALSIZE_COLUMN,
 							NULL);
 	gtk_tree_view_column_set_sort_column_id (column, TOTALSIZE_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
   	gtk_tree_view_append_column (treeview, column);
 	
 	/* Percentage */
@@ -407,6 +469,8 @@ add_columns (GtkTreeView *treeview)
 							NULL);
 
 	gtk_tree_view_column_set_title (column, (_("Percentage"))); 
+	gtk_tree_view_column_set_resizable(column, TRUE); 
+	gtk_tree_view_column_set_sort_column_id (column, PERCENTAGE_COLUMN);
     	gtk_tree_view_append_column(GTK_TREE_VIEW(treeview),column); 
 		
 	/* Elapse Time */
@@ -417,6 +481,7 @@ add_columns (GtkTreeView *treeview)
 							ELAPSETIME_COLUMN,
 							NULL);
 	gtk_tree_view_column_set_sort_column_id (column, ELAPSETIME_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 	/* REMAINTIME_COLUMN */
@@ -427,6 +492,7 @@ add_columns (GtkTreeView *treeview)
 														REMAINTIME_COLUMN,
 														NULL);
 	gtk_tree_view_column_set_sort_column_id (column, REMAINTIME_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 	/* Speed */
@@ -437,6 +503,7 @@ add_columns (GtkTreeView *treeview)
 														SPEED_COLUMN,
 														NULL);
 	gtk_tree_view_column_set_sort_column_id (column, SPEED_COLUMN);
+	gtk_tree_view_column_set_resizable(column, TRUE); 
 	gtk_tree_view_append_column (treeview, column);
 	
 }
@@ -514,6 +581,8 @@ gwget_gconf_notify_toolbar(GConfClient *client,
 	if (!strcmp(toolbar_setting,"text")) {
 		gtk_toolbar_set_style(GTK_TOOLBAR(toolbar),GTK_TOOLBAR_TEXT);
 	}
+
+	g_free(toolbar_setting);
 }
 
 static void 
@@ -653,6 +722,7 @@ gwget_remember_downloads(void)
 	GtkTreeIter iter;
 	gint i,length;
 	gboolean running = FALSE;
+	gchar down_size[2048];
 	
 	/* calculate the number of items in the treeview */
 	length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
@@ -685,10 +755,12 @@ gwget_remember_downloads(void)
 		key=g_strdup_printf("/apps/gwget2/downloads_data/%d/state",i);
 		gconf_client_set_int(gconf_client,key,gwgetdata->state,NULL);
 
-		key=g_strdup_printf ("/apps/gwget2/downloads_data/%d/total_size",i);
-		gconf_client_set_int (gconf_client,key,gwgetdata->total_size,NULL);		
+		key=g_strdup_printf ("/apps/gwget2/downloads_data/%d/file_size",i);
 		
-		if (gwgetdata->state==DL_RETRIEVING) 
+		g_snprintf(down_size,2047, "%" G_GINT64_FORMAT,gwgetdata->total_size);
+		gconf_client_set_string (gconf_client,key,down_size,NULL);		
+		
+		if (gwgetdata->log_tag != -1) 
 		{
 			running=TRUE;
 		}
