@@ -30,6 +30,8 @@
 #include "utils.h"
 #include "systray.h"
 
+static void inform_limit_speed_change(void);
+
 void 
 on_stop_button_clicked (GtkWidget *widget, gpointer data) 
 {
@@ -60,21 +62,7 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 	if (event->type == GDK_BUTTON_PRESS) {
 		event_button = (GdkEventButton *) event;
 		if (event->button==3 && gtk_tree_selection_get_selected (select, &model, &iter)) {
-			gwgetdata=gwget_data_get_selected();
-			if (gwget_data_run(gwgetdata)) {
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),TRUE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-				gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),TRUE);
-			} else {
-				if (gwgetdata->state==DL_COMPLETED) {
-					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),FALSE);
-					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"cancel_download"),FALSE);
-				} else {
-					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"continue_download"),TRUE);
-					gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_download"),FALSE);
-				}
-			}
+			gwget_data_set_menus(gwget_data_get_selected());
 			popup=glade_xml_get_widget(xml,"popup1");
 			gtk_menu_popup (GTK_MENU(popup), NULL, NULL, NULL, NULL, 
 							event_button->button, event_button->time);
@@ -93,8 +81,12 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 			if (gwgetdata->error) {
 				run_dialog(_("Error in download"),gwgetdata->error_msg);
 			} else {
-				uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
+				if (gwgetdata->recursive) {
+					uri = gnome_vfs_make_uri_from_input(gwgetdata->dir);
+				} else {
+					uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
                                                  GNOME_VFS_MAKE_URI_DIR_CURRENT);
+				}
 				if (!gnome_url_show (uri, &err)) {
 					run_dialog(_("Error opening file"),_("Couldn't open the file"));
 					return FALSE;
@@ -208,14 +200,14 @@ new_download(GwgetData* gwgetdata)
 	gdouble perc;
 	
 	gtk_list_store_append (GTK_LIST_STORE(model), &iter); 
-	size = g_strdup_printf ("%d kB", (gwgetdata->cur_size + 512) / 1024);
+	size = g_strdup_printf ("%d kB", (guint32)(gwgetdata->cur_size + 512) / 1024);
 	gtk_list_store_set (GTK_LIST_STORE(model), &iter,URL_COLUMN,gwgetdata->url,
 						CURRENTSIZE_COLUMN, size, 
 						FILENAME_COLUMN, gwgetdata->filename,
 	    				-1);
 	
 	
-	size = g_strdup_printf ("%d kB", (gwgetdata->total_size + 512) / 1024);
+	size = g_strdup_printf ("%d kB", (guint32)(gwgetdata->total_size + 512) / 1024);
 	gtk_list_store_set (GTK_LIST_STORE(model), &iter, TOTALSIZE_COLUMN, size, -1);
 	
 	perc = gwgetdata->total_size==0?0:((gdouble)gwgetdata->cur_size*100)/(gdouble)gwgetdata->total_size ;
@@ -229,10 +221,14 @@ new_download(GwgetData* gwgetdata)
 	
 	g_object_set_data(G_OBJECT(model),gwgetdata->url,gwgetdata);
 	
-	mime = (gchar *)gnome_vfs_mime_type_from_name(gwgetdata->local_filename);
 	theme = gtk_icon_theme_get_default ();
-	gwgetdata->icon_name = gnome_icon_lookup (theme, NULL, NULL, NULL, NULL,
-									mime, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+	if (!gwgetdata->recursive) {
+		mime = (gchar *)gnome_vfs_mime_type_from_name(gwgetdata->local_filename);
+		gwgetdata->icon_name = gnome_icon_lookup (theme, NULL, NULL, NULL, NULL,
+	 							mime, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+	} else {
+		gwgetdata->icon_name = g_strdup("gtk-refresh");
+	}
 	gtk_icon_size_lookup (GTK_ICON_SIZE_MENU, &width, &height);
 	width *= 2;
 	height *= 2;
@@ -434,10 +430,19 @@ on_pref_ok_button_clicked(GtkWidget *widget,gpointer data)
 	
 	/* Limit Speed */
 	limit_speed_check = glade_xml_get_widget (GLADE_XML(xml_pref), "limit_speed_check");
+	limit_speed_spin = glade_xml_get_widget (GLADE_XML(xml_pref), "limit_speed_spin");
+
+	if ( (count_download_in_progress()>0) && 
+	    ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_speed_check)) &&
+	    (gwget_pref.max_speed!=gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(limit_speed_spin)))) ||
+	    ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_speed_check)) && 
+	      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_speed_check))!=gwget_pref.limit_speed)))
+	{
+		inform_limit_speed_change();
+	}	
 	gwget_pref.limit_speed = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_speed_check));
 	gconf_client_set_bool(gconf_client,"/apps/gwget2/limit_speed",
 						  gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(limit_speed_check)),NULL);
-	limit_speed_spin = glade_xml_get_widget (GLADE_XML(xml_pref), "limit_speed_spin");
 	gwget_pref.max_speed = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(limit_speed_spin));
 	gconf_client_set_int(gconf_client,"/apps/gwget2/max_speed",
 						  gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(limit_speed_spin)),NULL);
@@ -696,6 +701,25 @@ on_view_toolbar_activate(GtkWidget *widget,gpointer data)
 	gconf_client_set_bool(gconf_client,"/apps/gwget2/view_toolbar",state,NULL);
 }
 
+void
+on_view_statusbar_activate(GtkWidget *widget, gpointer data)
+{
+	GtkWidget *statusbar, *menu_item;
+	gboolean state;
+
+	statusbar = glade_xml_get_widget (GLADE_XML(xml), "statusbar");
+	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_statusbar");
+	state = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item));
+	
+	if (!state) 
+	{
+		gtk_widget_hide(GTK_WIDGET(statusbar));
+	} else {
+		gtk_widget_show(GTK_WIDGET(statusbar));
+	}
+	gconf_client_set_bool(gconf_client,"/apps/gwget2/view_statusbar",state,NULL);
+}
+
 void 
 on_properties_activate(GtkWidget *widget, gpointer data)
 {
@@ -936,3 +960,119 @@ gint count_download_in_progress(void) {
 	}
 	return number;
 }
+
+/* Count all downloads */
+gint
+count_all_downloads(void)
+{
+	GtkTreeIter iter;
+	gint number;
+	gboolean iter_valid;
+
+	number=0;
+	for (iter_valid = gtk_tree_model_get_iter_first (model, &iter);iter_valid;iter_valid = gtk_tree_model_iter_next (model, &iter)) {
+		number++;
+	}
+	return number;
+}
+
+
+void
+on_download_menu_activate(void) 
+{
+	gwget_data_set_menus (gwget_data_get_selected());
+}
+
+static
+void inform_limit_speed_change(void)
+{
+	GtkWidget *dialog;
+
+	dialog = gtk_message_dialog_new (NULL,
+					GTK_DIALOG_MODAL,
+					GTK_MESSAGE_INFO,
+					GTK_BUTTONS_OK,
+					_("New download speed limit will apply only to new or restarted downloads"));
+	gtk_dialog_run (GTK_DIALOG (dialog));
+	gtk_widget_destroy (dialog);
+}
+
+void
+on_remove_download_activate (GtkWidget *widget, gpointer data)
+{
+	GwgetData *gwgetdata;
+	GtkTreeIter iter;
+	gchar *url;
+	gint response, length, i;
+
+	gwgetdata = gwget_data_get_selected();
+	if (gwgetdata != NULL) {
+		response = run_dialog(_("Remove this download ?"),_("Really remove this download from the list?"));
+		if (response == GTK_RESPONSE_OK) {
+			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
+			gtk_tree_model_get_iter_root(model,&iter);
+			for (i=0;i<length;i++) {
+				gtk_tree_model_get (model, &iter, URL_COLUMN, &url, -1);
+				if (gwgetdata==g_object_get_data(G_OBJECT(model),url)) {
+					gtk_list_store_remove(GTK_LIST_STORE(model),&iter);
+					downloads=g_list_remove(downloads,gwgetdata);
+				} else {
+					gtk_tree_model_iter_next(model,&iter);
+				}
+			}
+		}
+	}
+}
+
+void
+on_open_download_activate (GtkWidget *widget, gpointer data)
+{
+	GwgetData *gwgetdata;
+	gchar *uri;
+	GError *err = NULL;
+
+	gwgetdata=gwget_data_get_selected();
+	g_return_if_fail (gwgetdata!=NULL);
+
+	uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
+							GNOME_VFS_MAKE_URI_DIR_CURRENT);
+
+	if (!gnome_url_show (uri, &err)) {
+		run_dialog(_("Error opening file"),_("Couldn't open the file"));
+		return;
+	}
+}
+
+void
+on_open_directory_activate (GtkWidget *widget, gpointer data)
+{
+	GwgetData *gwgetdata;
+	gchar *uri;
+	GError *err = NULL;
+
+	gwgetdata=gwget_data_get_selected();
+	g_return_if_fail (gwgetdata!=NULL);
+
+
+	uri = gnome_vfs_make_uri_from_input(gwgetdata->dir);
+
+	if (!gnome_url_show (uri, &err)) {
+		run_dialog(_("Error opening file"),_("Couldn't open the folder"));
+		return;
+	}
+}
+
+void
+on_file_menuitem_activate (GtkWidget *widget, gpointer data)
+{
+
+	if (count_all_downloads()==0)
+	{
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_all_menuitem"), FALSE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"resume_all_menuitem"), FALSE);
+	} else {
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"pause_all_menuitem"), TRUE);
+		gtk_widget_set_sensitive(glade_xml_get_widget(xml,"resume_all_menuitem"), TRUE);
+	}
+
+}
